%%% File    : locker_sup.erl
%%% Author  : Clara Benac Earle <clara@bacardi>
%%% Purpose : fourth version of the locker (shared locks) with supervision
%%% Created : 12 Apr 2001 by Clara Benac Earle <clara@bacardi>
%%% Modified: 08 Jun 2001 by Thomas Arts <thomas@cslab.ericsson.se>

-module(locker_sup).
-author('clara@bacardi').

-behaviour(supervisor).

-export([start/1]).
-export([init/1]).

start(ResTypes) ->
    supervisor:start_link({local, ?MODULE}, ?MODULE, ResTypes).

init(ResTypes) ->
    Resources = 
      lists:foldl(fun({Rs,T},UniqRes) ->
               lists:foldl(fun(R,Res) ->
                        adduniq(R,Res)
                     end,UniqRes,Rs)
            end,[],ResTypes),
    {ok,{{one_for_one,2,60}, 
          [{locker,{locker,start_link,[Resources]},
            permanent, 2000, worker, dynamic},
           {client_supervisor, 
            {client_sup, start_link, [locker,ResTypes]},
            permanent, infinity, supervisor,dynamic}]}}.


adduniq(Element,[]) ->
    [Element];
adduniq(Element,[Element|Tail]) ->
    [Element|Tail];
adduniq(Element,[Head|Tail]) ->
    [Head|adduniq(Element,Tail)].

